%
%This RMA SAR algorithm was written by Gregory L. Charvat as part of his dissertation:
%G. L. Charvat, ``A Low-Power Radar Imaging System," Ph.D. dissertation,
%Dept. of Electrical and Computer Engineering, Michigan State University,
%East Lansing, MI, 2007.
%
%Please cite appropriately.
%
%This algorithm was based on:
%Range Migration Algorithm from ch 10 of Spotlight Synthetic Aperture Radar
%Signal Processing Algorithms, Carrara, Goodman, and Majewski
%
%The radar system used here is from the following sources:
%G. L. Charvat, L. C. Kempel, E. J. Rothwell, C. Coleman, and E. L. Mokole, ``A through-dielectric radar imaging system," IEEE Transactions on Antennas and Propagation, vol. 58, Issue 8, pp. 2594-2603, August, 2010.
%G. L. Charvat, ?A low-power radar imaging system,? The Boston Chapter of the IEEE APS Society, December 11, 2007
%G. L. Charvat, ``A Low-Power Radar Imaging System," Ph.D. dissertation, Dept. of Electrical and Computer Engineering, Michigan State University, East Lansing, MI, 2007.
%
%Radar specifications:
% LFM chirp from 1.926 GHz - 4.069 GHz in 10 mS
% TX power = 10 dBm
% TX and RX beamwidth estimated to be 15 deg E plane, 30 deg in H plane
% rail SAR length is 96"
% cross-range sample spacing = 2"
%
%Data:  Target scene of a group of 6" tall vertical carriage bolts in a block
%S configureation.  Acquired in October of 2006 while in graduate school 
%as part of the Electromagnetics Group at Michigan State University

clear all;
c = 3E8; %(m/s) speed of light

%*********************************************************************
%load data
load rarray_3_25_13_evening_demo.mat %load variable sif %for image data
%will load background, cal, and data s

%load background data
sback = background; %load variable sif %for background subtraction cal data


%************************************************************************
%callibration
%load callibration file to standard target
s_cal = cal;
%load background data for cal to standard target
s_cal = s_cal - background; %perform background subtraction
cal = s_cal;
cal_range =11*.3048;

%compute number of frames, where each frame is 250 samples long
n = size(s,2)/250;

vidObj = VideoWriter('test','Motion JPEG AVI');
open(vidObj);

%loop through each frame, make image, display data
for nn = 1:(n-1)

%peel off one frame at index ii
%nn = 20; %good image of 2 cylinders
sframe = s(:,250*nn+1:250*(nn+1));
nn

%*********************************************************************
%perform background subtraction
sif = sframe-sback; %perform coherent background subtraction
%sif = sif_sub; %image just the background
%sif = s; %image without background subtraction

%CCD between frames
%sback = sframe;

%***********************************************************************
%radar parameters
fc = (4.069E9 - 1.926E9)/2 + 1.926E9; %(Hz) center radar frequency
B = (4.069E9 - 1.926E9); %(hz) bandwidth
cr = 2E9/10E-3; %(Hz/sec) chirp rate
Tp = 10E-3; %(sec) pulse width
Xa = 0; %(m) beginning of new aperture length
delta_x = (2*1/12)*0.3048; %(m) 2 inch antenna spacing
L = delta_x*(size(sif,1)); %(m) aperture length
Xa = linspace(-L/2, L/2, (L/delta_x))+delta_x; %(m) cross range position of radar on aperture L
%adding a delta_x to account for a 2 inch offset in phase centers wrt cal
%location, which is assumed to be aligned with array-frame center
Za = 0;
t = linspace(0, Tp, size(sif,2)); %(s) fast time, CHECK SAMPLE RATE
Kr = linspace(((4*pi/c)*(fc - B/2)), ((4*pi/c)*(fc + B/2)), (size(t,2)));

Rs = sqrt(Xa.^2 + cal_range^2); %(m) y coordinate to scene center (down range), make this value equal to distance to cal target
Ya = Rs; %THIS IS VERY IMPORTANT, SEE GEOMETRY FIGURE 10.6

%calculate ideal cal target parameters
%target parameters, 3 targets
at1 = 1; %amplitude of cal target
xt1 = 0;
yt1 = cal_range; %(m) distance to cal target
zt1 = 0;

%Rt and Rb for 1 2D cal target according to equation 10.26
for ii = 1:size(Kr,2); %step thru each time step to find phi_if
    for jj = 1:size(Xa,2) %step thru each azimuth step
        phi_if1(jj,ii) = Kr(ii)*(-1*Rs(jj));
    end
end
cal_theory = at1*exp(-j*phi_if1);

%calculate the calibration factor
cf = cal_theory./(cal);

%apply the cal data
sif = sif.*cf; %turn off cal by commenting here

%Normalize amplitude to middle of aperture
ref = abs(sif(22,:));
for ii = 1:size(sif,1)
    sif(ii,:) = ref.*sif(ii,:)./abs(sif(ii,:));
end

%Save background subtracted and callibrated data to pass onto IFP
clear Rs;
Rs = cal_range;
save sif sif delta_x Rs Kr Xa cal_range;
%clear all;

%run IFP
SBAND_RMA_IFP;

%grab a frame for each image
currFrame = getframe;

%need to set frame rate
for zz = 1:12
    writeVideo(vidObj,currFrame);
end

end

%close video file
close(vidObj);